import asyncio
import websockets
import json

async def hello():
    async with websockets.connect("ws://localhost:8000/ws/notifications/72dcdc3dd3a292d1846ad30f88befcb6b7691edd/") as websocket:
        data = await websocket.recv()
        print(data)
        notif = json.loads(data)
        await websocket.send(json.dumps({'type':'markread', 'notification': notif['id']}))
        await websocket.recv()

        all_notifs = await websocket.send(json.dumps({'type': 'unread'}))
        print(await websocket.recv())

asyncio.run(hello())


# Generated by Django 3.2.15 on 2023-05-11 23:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('magic_notifier', '0003_auto_20220914_1756'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='subject',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



import os

from channels.auth import AuthMiddlewareStack
from channels.routing import ProtocolTypeRouter, URLRouter
from channels.security.websocket import AllowedHostsOriginValidator
from django.core.asgi import get_asgi_application
from django.urls import path

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "example.settings")
# Initialize Django ASGI application early to ensure the AppRegistry
# is populated before importing code that may import ORM models.
django_asgi_app = get_asgi_application()

from magic_notifier.consumers import PushNotifConsumer

application = ProtocolTypeRouter({
    # Django's ASGI application to handle traditional HTTP requests
    "http": django_asgi_app,

    # WebSocket chat handler
    "websocket": URLRouter([
                path("ws/notifications/<str:token>/", PushNotifConsumer.as_asgi()),
            ])
    }
)


"""
Django settings for example project.

Generated by 'django-admin startproject' using Django 2.2.17.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'm#*@g4n)s5zt6ug!4(16x_j613i1(ft+jkq#r8*c@74k@f^iw9'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'channels',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'core',
    'magic_notifier',
    'rest_framework',
    'rest_framework.authtoken',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'example.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'example.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = '/static/'

NOTIFIER = {
    "SMTP": {
        "default": {
            "HOST": "localhost",
            "PORT": 587,
            "USE_TLS": True,
            "USE_SSL": False,
            "USER": "root@localhost",
            "FROM": "Root <root@localhost>",
            "PASSWORD": "********",
            "CLIENT": "magic_notifier.email_clients.django_email.DjangoEmailClient",
        }
    },
    "USER_FROM_WS_TOKEN_FUNCTION": 'magic_notifier.utils.get_user_from_ws_token'
}

ASGI_APPLICATION = 'example.asgi.application'

LOGS_FORMAT = (
    "[%(asctime)s] %(levelname)s[%(name)s] %(message)s - %(pathname)s#lines-%(lineno)s"
)

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "standard": {
            "format": LOGS_FORMAT,
            "datefmt": "%d/%b/%Y %H:%M:%S",
        },
    },
    "handlers": {
        "default": {
            "level": "DEBUG" if DEBUG else "INFO",
            "class": "logging.StreamHandler",
            "formatter": "standard",
        },
    },
    "loggers": {
        "django": {
            "handlers": ["default"],
            "level": "ERROR",
            "propagate": True,
        },
        "": {
            "handlers": ["default"],  # Logstash disabled
            "level": "DEBUG",
            "propagate": True,
        },
        #"django.request": {"handlers": ["default"], "level": "DEBUG"},
    },
}

CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels.layers.InMemoryChannelLayer"
    }
}


import json
import time
from pathlib import Path
from unittest import mock
from unittest.mock import patch

import requests
from django.contrib.auth import get_user_model
from django.core import mail
from django.core.management import call_command
from django.template.loader import render_to_string
from django.test import TestCase, override_settings, LiveServerTestCase

from magic_notifier.models import NotifyProfile, Notification
from magic_notifier.notifier import notify
from magic_notifier.pusher import Pusher
from magic_notifier.utils import NotificationBuilder
from magic_notifier.serializers import NotificationSerializer
User = get_user_model()


class EmailTestCase(TestCase):
    """Class to test emails sending"""

    @classmethod
    def setUpClass(cls):
        User.objects.create(username="user1", email="user1@localhost")
        User.objects.create(username="user2", email="user2@localhost")
        User.objects.create(username="user3", email="user3@localhost")

        User.objects.create(username="user4", email="user4@localhost", is_staff=True)
        User.objects.create(username="user5", email="user5@localhost", is_staff=True)

        User.objects.create(username="user6", email="user6@localhost",
            is_superuser=True, is_staff=True)
        return super().setUpClass()

    @classmethod
    def tearDownClass(cls):
        User.objects.all().delete()
        return super().tearDownClass()

    def test_simple_with_user(self):
        user = User(email="testuser@localhost", username="testuser")

        subject = "Test magic notifier"
        notify(["email"], subject, [user], final_message="Nice if you get this")

        self.assertGreater(len(mail.outbox), 0) # type: ignore
        first_message = mail.outbox[0] # type: ignore
        self.assertEqual(first_message.to, [user.email])
        self.assertEqual(first_message.subject, subject)

    def test_simple_with_user_threaded(self):
        user = User(email="testuser@localhost", username="testuser")

        subject = "Test magic notifier"
        notify(["email"], subject, [user], final_message="Nice if you get this",
            threaded=True)
        time.sleep(2)

        self.assertGreater(len(mail.outbox), 0) # type: ignore
        first_message = mail.outbox[0] # type: ignore
        self.assertEqual(first_message.to, [user.email])
        self.assertEqual(first_message.subject, subject)

    def test_simple_direct_email(self):
        subject = "Test magic notifier"
        notify(["email"], subject, ["testuser@localhost"], final_message="Nice if you get this")

        self.assertGreater(len(mail.outbox), 0) # type: ignore
        first_message = mail.outbox[0] # type: ignore
        self.assertEqual(first_message.to, ["testuser@localhost"])
        self.assertEqual(first_message.subject, subject)

    def test_template_html_txt_with_user(self):
        user = User(email="testuser@localhost", username="testuser")

        subject = "Test magic notifier"
        notify(["email"], subject, [user], template='base')

        self.assertGreater(len(mail.outbox), 0) # type: ignore
        first_message = mail.outbox[0] # type: ignore
        self.assertEqual(first_message.to, [user.email])
        self.assertEqual(first_message.subject, subject)
        self.assertEqual(len(first_message.alternatives), 1)

    def test_template_txt_with_user(self):
        user = User(email="testuser@localhost", username="testuser")

        subject = "Test magic notifier"
        notify(["email"], subject, [user], template='hello')

        self.assertGreater(len(mail.outbox), 0) # type: ignore
        first_message = mail.outbox[0] # type: ignore
        self.assertEqual(first_message.to, [user.email])
        self.assertEqual(first_message.subject, subject)
        self.assertEqual(len(first_message.alternatives), 0)

    def test_template_not_exist_with_user(self):
        user = User(email="testuser@localhost", username="testuser")

        subject = "Test magic notifier"
        notify(["email"], subject, [user], template='notexist')

        self.assertEqual(len(mail.outbox), 0) # type: ignore

    def test_command_test_email_template(self):
        call_command('test_email_template', 'hello', 'testuser@localhost')

        self.assertGreater(len(mail.outbox), 0) # type: ignore
        first_message = mail.outbox[0] # type: ignore
        self.assertEqual(first_message.to, ['testuser@localhost'])
        self.assertEqual(first_message.subject, 'Testing email template')
        self.assertEqual(len(first_message.alternatives), 0)

    def test_template_txt_with_user_with_files_filename(self):
        user = User(email="testuser@localhost", username="testuser")

        subject = "Test magic notifier"
        notify(["email"], subject, [user], template='hello',
            files=[str(Path(__file__).parent / "models.py")])

        self.assertGreater(len(mail.outbox), 0) # type: ignore
        first_message = mail.outbox[0] # type: ignore
        self.assertEqual(first_message.to, [user.email])
        self.assertEqual(first_message.subject, subject)
        self.assertEqual(len(first_message.alternatives), 0)
        self.assertGreater(len(first_message.attachments), 0)

    def test_template_txt_with_user_with_files_tuple_string(self):
        user = User(email="testuser@localhost", username="testuser")

        subject = "Test magic notifier"
        notify(["email"], subject, [user], template='hello',
            files=[("test.txt", "")])

        self.assertGreater(len(mail.outbox), 0) # type: ignore
        first_message = mail.outbox[0] # type: ignore
        self.assertEqual(first_message.to, [user.email])
        self.assertEqual(first_message.subject, subject)
        self.assertEqual(len(first_message.alternatives), 0)
        self.assertEqual(len(first_message.attachments), 0) # it failed

    def test_template_txt_with_user_with_files_tuple_filelike(self):
        user = User(email="testuser@localhost", username="testuser")

        subject = "Test magic notifier"
        with open(str(Path(__file__).parent / "models.py")) as fp:
            notify(["email"], subject, [user], template='hello',
                files=[("models.py", fp)])

            self.assertGreater(len(mail.outbox), 0) # type: ignore
            first_message = mail.outbox[0] # type: ignore
            self.assertEqual(first_message.to, [user.email])
            self.assertEqual(first_message.subject, subject)
            self.assertEqual(len(first_message.alternatives), 0)
            self.assertGreater(len(first_message.attachments), 0)

    def test_template_txt_with_user_with_files_bad(self):
        user = User(email="testuser@localhost", username="testuser")

        subject = "Test magic notifier"
        with open(str(Path(__file__).parent / "models.py")) as fp:
            notify(["email"], subject, [user], template='hello',
                files=[1])

            self.assertGreater(len(mail.outbox), 0) # type: ignore
            first_message = mail.outbox[0] # type: ignore
            self.assertEqual(first_message.to, [user.email])
            self.assertEqual(first_message.subject, subject)
            self.assertEqual(len(first_message.alternatives), 0)
            self.assertEqual(len(first_message.attachments), 0)

    def test_template_txt_with_user_with_files_filelike(self):
        user = User(email="testuser@localhost", username="testuser")

        subject = "Test magic notifier"
        with open(str(Path(__file__).parent / "models.py")) as fp:
            notify(["email"], subject, [user], template='hello',
                files=[fp])

            self.assertGreater(len(mail.outbox), 0) # type: ignore
            first_message = mail.outbox[0] # type: ignore
            self.assertEqual(first_message.to, [user.email])
            self.assertEqual(first_message.subject, subject)
            self.assertEqual(len(first_message.alternatives), 0)
            self.assertGreater(len(first_message.attachments), 0)

    def test_simple_with_string_all(self):
        subject = "Test magic notifier"
        notify(["email"], subject, "all", final_message="Nice if you get this")

        self.assertEqual(len(mail.outbox), 6) # type: ignore

    def test_simple_with_string_staff(self):
        subject = "Test magic notifier"
        notify(["email"], subject, "staff", final_message="Nice if you get this")

        self.assertEqual(len(mail.outbox), 3) # type: ignore

    def test_simple_with_string_admins(self):
        subject = "Test magic notifier"
        notify(["email"], subject, "admins", final_message="Nice if you get this")

        self.assertEqual(len(mail.outbox), 1) # type: ignore

    def test_simple_with_string_all_minus_admins(self):
        subject = "Test magic notifier"
        notify(["email"], subject, "all-admins", final_message="Nice if you get this")

        self.assertEqual(len(mail.outbox), 5) # type: ignore

    def test_simple_with_string_all_minius_staff(self):
        subject = "Test magic notifier"
        notify(["email"], subject, "all-staff", final_message="Nice if you get this")

        self.assertEqual(len(mail.outbox), 3) # type: ignore

    def test_simple_with_string_unknown(self):
        subject = "Test magic notifier"
        self.assertRaises(ValueError, notify, ["email"], subject, "unknown", final_message="Nice if you get this")

    def test_unknown_method(self):
        subject = "Test magic notifier"
        notify(["unknown"], subject, "all-staff", final_message="Nice if you get this")
        self.assertEqual(len(mail.outbox), 0) # type: ignore


sms_outbox = []

class Sms:

    def __init__(self, number, message):
        self.number = number
        self.message = message

def send_to_sms_outbox(*args, **kwargs):
    print(*args)
    try:
        url = args[-1]
    except:
        url = None
    params = kwargs.get('params')
    data = kwargs.get('data')
    if url:
        if url == 'https://smsvas.com/bulk/public/index.php/api/v1/sendsms':
            data = kwargs['json']
            sms_outbox.append(Sms(data['mobiles'], data['sms']))
            res = requests.Response()
            res.status_code = 200
            return res
        elif url == 'http://cheapglobalsms.com/api_v1':
            params = kwargs['params']
            sms_outbox.append(Sms(params['recipients'], params['message']))
        else:
            sms_outbox.append(Sms(data['To'], data['Body']))
            from twilio.http.response import Response

            return mock.MagicMock(spec=Response, status_code=200,
                text=json.dumps({
                          "account_sid": "ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
                          "api_version": "2010-04-01",
                          "body": "Hi there",
                          "date_created": "Thu, 30 Jul 2015 20:12:31 +0000",
                          "date_sent": "Thu, 30 Jul 2015 20:12:33 +0000",
                          "date_updated": "Thu, 30 Jul 2015 20:12:33 +0000",
                          "direction": "outbound-api",
                          "error_code": None,
                          "error_message": None,
                          "from": "+15017122661",
                          "messaging_service_sid": "MGXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
                          "num_media": "0",
                          "num_segments": "1",
                          "price": None,
                          "price_unit": None,
                          "sid": "SMXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
                          "status": "sent",
                          "subresource_uris": {
                            "media": "/2010-04-01/Accounts/ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Messages/SMXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Media.json"
                          },
                          "to": "+15558675310",
                          "uri": "/2010-04-01/Accounts/ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX/Messages/SMXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX.json"
                        }))


def notifier_settings_for_global_cheap(*args, **kwargs):
    if args[0] == "SMS::DEFAULT_GATEWAY":
        return "CGS"
    elif args[0] == "SMS":
        return {
                    "GATEWAYS": {
                        "CGS": {
                            "CLIENT": "magic_notifier.sms_clients.cgsms_client.CGSmsClient",
                            "SUB_ACCOUNT": "sub_account",
                            "SUB_ACCOUNT_PASSWORD": "sub_account_password"
                        }
                    }
                }
    elif args[0] == "GET_USER_NUMBER":
        return "magic_notifier.utils.get_user_number"


def notifier_settings_for_nexa(*args, **kwargs):
    if args[0] == "SMS::DEFAULT_GATEWAY":
        return "NEXA"
    elif args[0] == "SMS":
        return {
                    "GATEWAYS": {
                        "NEXA": {
                            "CLIENT": "magic_notifier.sms_clients.nexa_client.NexaSmsClient",
                            "EMAIL": "sub_account",
                            "PASSWORD": "sub_account_password",
                            "SENDERID": "senderid"
                        }
                    }
                }
    elif args[0] == "GET_USER_NUMBER":
        return "magic_notifier.utils.get_user_number"


def notifier_settings_for_twilio(*args, **kwargs):
    if args[0] == "SMS::DEFAULT_GATEWAY":
        return "TWILIO"
    elif args[0] == "SMS":
        return {
                    "GATEWAYS": {
                        "TWILIO": {
                            "CLIENT": "magic_notifier.sms_clients.twilio_client.TwilioClient",
                            "ACCOUNT": "sub_account",
                            "TOKEN": "token",
                            "FROM_NUMBER": "from_number"
                        }
                    }
                }
    elif args[0] == "GET_USER_NUMBER":
        return "magic_notifier.utils.get_user_number"


class SmsTestCase(TestCase):

    @patch('magic_notifier.smser.get_settings', side_effect=notifier_settings_for_global_cheap)
    @patch('magic_notifier.sms_clients.cgsms_client.requests.get', side_effect=send_to_sms_outbox)
    def test_global_cheap_sms_client(self, mock_get_request, mock_get_settings):
        NOTIFIER = {
            "SMS":{
                "GATEWAYS": {
                    "CGS": {
                        "CLIENT": "magic_notifier.sms_clients.cgsms_client.CGSmsClient",
                        "SUB_ACCOUNT": "sub_account",
                        "SUB_ACCOUNT_PASSWORD": "sub_account_password"
                    }
                },
                "DEFAULT_GATEWAY": "CGS"
            }
        }

        with self.settings(NOTIFIER=NOTIFIER):
            user = User.objects.create(email="testuser@localhost", username="testuser")
            not_profile = NotifyProfile.objects.create(phone_number="+237600000000",
                user=user)

            subject = "Test magic notifier"
            notify(["sms"], subject, [user], final_message="Nice if you get this")

            self.assertGreater(len(sms_outbox), 0) # type: ignore
            first_message = sms_outbox[0] # type: ignore
            self.assertEqual(first_message.number, not_profile.phone_number)
            self.assertEqual(first_message.message, "Nice if you get this")

    @patch('magic_notifier.smser.get_settings', side_effect=notifier_settings_for_twilio)
    @patch('twilio.http.http_client.TwilioHttpClient.request', side_effect=send_to_sms_outbox)
    def test_twilio_sms_client(self, mock_get_request, mock_get_settings):
        NOTIFIER = {
            "SMS": {
                "GATEWAYS": {
                    "TWILIO": {
                        "CLIENT": "magic_notifier.sms_clients.twilio_client.TwilioClient",
                        "ACCOUNT": "sub_account",
                        "TOKEN": "token",
                        "FROM_NUMBER": "from_number"
                    }
                },
                "DEFAULT_GATEWAY": "TWILIO"
            }
        }

        with self.settings(NOTIFIER=NOTIFIER):
            user = User.objects.create(email="testuser@localhost", username="testuser")
            not_profile = NotifyProfile.objects.create(phone_number="+237600000000",
                user=user)

            subject = "Test magic notifier"
            notify(["sms"], subject, [user], final_message="Nice if you get this")

            self.assertGreater(len(sms_outbox), 0) # type: ignore
            first_message = sms_outbox[0] # type: ignore
            self.assertEqual(first_message.number, not_profile.phone_number)
            self.assertEqual(first_message.message, "Nice if you get this")

    @patch('magic_notifier.smser.get_settings', side_effect=notifier_settings_for_nexa)
    @patch('magic_notifier.sms_clients.cgsms_client.requests.post', side_effect=send_to_sms_outbox)
    def test_nexa_sms_client(self, mock_get_request, mock_get_settings):
        NOTIFIER = {
            "SMS": {
                "GATEWAYS": {
                    "NEXA": {
                        "CLIENT": "magic_notifier.sms_clients.nexa_client.NexaSmsClient",
                        "EMAIL": "sub_account",
                        "PASSWORD": "sub_account_password",
                        "SENDERID": "senderid"
                    }
                },
                "DEFAULT_GATEWAY": "NEXA"
            }
        }

        with self.settings(NOTIFIER=NOTIFIER):
            user = User.objects.create(email="testuser@localhost", username="testuser")
            not_profile = NotifyProfile.objects.create(phone_number="+237600000000",
                                                       user=user)

            subject = "Test magic notifier"
            notify(["sms"], subject, [user], final_message="Nice if you get this")

            self.assertGreater(len(sms_outbox), 0)  # type: ignore
            first_message = sms_outbox[0]  # type: ignore
            self.assertEqual(first_message.number, not_profile.phone_number)
            self.assertEqual(first_message.message, "Nice if you get this")


class PushNotificationTestCase(TestCase):

    def test_load_json(self):
        ctx = {
            'text': 'yes',
            'actions': [
                {
                    'text': 'wow',
                    'url': 'accept',
                    'method': 'post'
                },
                {
                    'text': 'meow',
                    'url': 'deny',
                    'method': 'get'
                }
            ],
            'data': {
                'love': 'you',
                'hate': 'no-one'
            }
        }
        push_content = render_to_string(f"notifier/base/push.json", ctx)
        print(push_content)
        self.assertIsInstance(json.loads(push_content), dict)

    def test_notification_builder_class(self):
        notif = NotificationBuilder("just a test").text("This is just a test").type('test', 'test_sub')\
            .link("http://lol").save()
        self.assertIsInstance(notif, Notification)
        seria = NotificationSerializer(instance=notif)
        print(notif)
        print(seria.data)


class LivePushNotificationTestCase(LiveServerTestCase):
    port = 8001

    def test_pusher_class(self):
        user = User.objects.create_user('testuser')
        NotifyProfile.objects.create(user=user)
        pusher = Pusher("just a test", [user], 'base', {'data': {'love': 'you', 'hate': 'no-one'},
                                                        'actions': [{'text':'accept', 'method':'post', 'url': 'http://'}]})
        notif = pusher.send()
        self.assertIsInstance(notif, Notification)
        seria = NotificationSerializer(instance=notif)
        print(notif)
        print(seria.data)
